require "spec_helper"

describe ActionServer do
  def app
    ActionServer
  end

  let(:user_id) { '54aa4fd5e7998536b8fa78e1' }
  let(:email_campaign_id) { '54a680d56d616927d3020000' }
  let(:ip) { '127.0.0.1' }
  let(:additional_params) { nil }
  let(:message) {
    MultiJson.dump(
      ip:                ip,
      url:               url,
      user_id:           user_id,
      email_campaign_id: email_campaign_id
    )
  }

  describe "get /click/:data" do
    let(:query_string) { 'url=http://example.com&user_id=123' }
    let(:encrypted_query_string) { UrlEncryptor.encrypt_query(query_string) }

    before do
      any_instance_of(ActionServer) do |klass|
        stub(klass).publish
      end
    end

    it "return successful response" do
      get "/click/#{ encrypted_query_string }"

      expect(last_response).to be_redirect
    end
  end

  describe "get /open/:data" do
    let(:query_string) { 'email=qwe@qwe.com&user_id=123' }
    let(:encrypted_query_string) { UrlEncryptor.encrypt_query(query_string) }

    before do
      any_instance_of(ActionServer) do |klass|
        stub(klass).publish
      end
    end

    it "return successful response" do
      get "/open/#{ encrypted_query_string }"

      expect(last_response).to be_successful
    end
  end

  describe "get /proxy/:data" do
    before { ActionServer.cache_dir = '/tmp/' }

    context 'with exists img' do
      let(:url) { 'http://image.lascana.ru/pool/formatz/11570525.jpg' }
      let(:cache_file_path) { '/tmp/e4d421d6763ffc0df34706c403440a2b.cache' }
      let(:query_string) { "url=#{ url }&user_id=123" }
      let(:encrypted_query_string) { UrlEncryptor.encrypt_query(query_string) }

      it "return successful response" do
        get "/proxy/#{ encrypted_query_string }"

        expect(last_response).to be_successful
        expect(File.exists?(cache_file_path)).to eq(true)
      end
    end

    context 'without exists img' do
      let(:url) { 'http://image.lascana.ru/qweqwe.jpg' }
      let(:query_string) { "url=#{ url }&user_id=123" }
      let(:encrypted_query_string) { UrlEncryptor.encrypt_query(query_string) }

      it "return failure response" do
        get "/proxy/#{ encrypted_query_string }"

        expect(last_response.status).to eq(404)
      end
    end
  end
end